"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTypeName = exports.getNamedArguments = exports.FunctionNodeParser = void 0;
const typescript_1 = __importDefault(require("typescript"));
const FunctionType_1 = require("../Type/FunctionType");
const NeverType_1 = require("../Type/NeverType");
const DefinitionType_1 = require("../Type/DefinitionType");
const ObjectType_1 = require("../Type/ObjectType");
const nodeKey_1 = require("../Utils/nodeKey");
class FunctionNodeParser {
    constructor(childNodeParser, functions) {
        this.childNodeParser = childNodeParser;
        this.functions = functions;
    }
    supportsNode(node) {
        return (node.kind === typescript_1.default.SyntaxKind.FunctionType ||
            node.kind === typescript_1.default.SyntaxKind.FunctionExpression ||
            node.kind === typescript_1.default.SyntaxKind.ArrowFunction ||
            node.kind === typescript_1.default.SyntaxKind.FunctionDeclaration);
    }
    createType(node, context) {
        if (this.functions === "hide") {
            return new NeverType_1.NeverType();
        }
        const name = getTypeName(node);
        const func = new FunctionType_1.FunctionType(node, getNamedArguments(this.childNodeParser, node, context));
        return name ? new DefinitionType_1.DefinitionType(name, func) : func;
    }
}
exports.FunctionNodeParser = FunctionNodeParser;
function getNamedArguments(childNodeParser, node, context) {
    if (node.parameters.length === 0) {
        return undefined;
    }
    const parameterTypes = node.parameters.map((parameter) => {
        return childNodeParser.createType(parameter, context);
    });
    return new ObjectType_1.ObjectType(`object-${(0, nodeKey_1.getKey)(node, context)}`, [], parameterTypes.map((parameterType, index) => {
        const required = node.parameters[index].questionToken ? false : !node.parameters[index].initializer;
        return new ObjectType_1.ObjectProperty(node.parameters[index].name.getText(), parameterType, required);
    }), false);
}
exports.getNamedArguments = getNamedArguments;
function getTypeName(node) {
    var _a;
    if (typescript_1.default.isArrowFunction(node) || typescript_1.default.isFunctionExpression(node) || typescript_1.default.isFunctionTypeNode(node)) {
        const parent = node.parent;
        if (typescript_1.default.isVariableDeclaration(parent)) {
            return parent.name.getText();
        }
    }
    if (typescript_1.default.isFunctionDeclaration(node)) {
        return (_a = node.name) === null || _a === void 0 ? void 0 : _a.getText();
    }
    return undefined;
}
exports.getTypeName = getTypeName;
//# sourceMappingURL=FunctionNodeParser.js.map