"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.extractLiterals = void 0;
const UnknownTypeError_1 = require("../Error/UnknownTypeError");
const AliasType_1 = require("../Type/AliasType");
const BooleanType_1 = require("../Type/BooleanType");
const DefinitionType_1 = require("../Type/DefinitionType");
const EnumType_1 = require("../Type/EnumType");
const LiteralType_1 = require("../Type/LiteralType");
const UnionType_1 = require("../Type/UnionType");
const derefType_1 = require("./derefType");
function* _extractLiterals(type) {
    if (!type) {
        return;
    }
    const dereffedType = (0, derefType_1.derefAnnotatedType)(type);
    if (dereffedType instanceof LiteralType_1.LiteralType) {
        yield dereffedType.getValue().toString();
        return;
    }
    if (dereffedType instanceof UnionType_1.UnionType || dereffedType instanceof EnumType_1.EnumType) {
        for (const t of dereffedType.getTypes()) {
            yield* _extractLiterals(t);
        }
        return;
    }
    if (dereffedType instanceof AliasType_1.AliasType || dereffedType instanceof DefinitionType_1.DefinitionType) {
        yield* _extractLiterals(dereffedType.getType());
        return;
    }
    if (dereffedType instanceof BooleanType_1.BooleanType) {
        yield* _extractLiterals(new UnionType_1.UnionType([new LiteralType_1.LiteralType("true"), new LiteralType_1.LiteralType("false")]));
        return;
    }
    throw new UnknownTypeError_1.UnknownTypeError(dereffedType);
}
function extractLiterals(type) {
    return [..._extractLiterals(type)];
}
exports.extractLiterals = extractLiterals;
//# sourceMappingURL=extractLiterals.js.map